use crate::{
	content, physics::Pfloat, render::SpriteTexture, Doodad, Sprite, Spriteable, STARFIELD_COUNT,
	STARFIELD_PARALLAX_MAX, STARFIELD_PARALLAX_MIN, STARFIELD_SIZE,
};
use cgmath::{Point2, Vector2};
use rand::{self, Rng};

pub struct StarfieldStar {
	/// Star coordinates, in world space.
	/// These are relative to the center of a starfield tile.
	pub pos: Point2<Pfloat>,

	// TODO: z-coordinate?
	pub parallax: Pfloat,
	pub size: Pfloat,

	/// Color/brightness variation.
	/// See shader.
	pub tint: Vector2<Pfloat>,
}

pub struct System {
	pub name: String,
	bodies: Vec<Doodad>,
	pub starfield: Vec<StarfieldStar>,
}

impl System {
	pub fn new(ct: &content::system::System) -> Self {
		let mut rng = rand::thread_rng();
		let sz = STARFIELD_SIZE as f32 / 2.0;
		let mut s = System {
			name: ct.name.clone(),
			bodies: Vec::new(),
			starfield: (0..STARFIELD_COUNT)
				.map(|_| StarfieldStar {
					pos: Point2 {
						x: rng.gen_range(-sz..=sz),
						y: rng.gen_range(-sz..=sz),
					},
					parallax: rng.gen_range(STARFIELD_PARALLAX_MIN..STARFIELD_PARALLAX_MAX),
					size: rng.gen_range(0.2..0.8), // TODO: configurable
					tint: Vector2 {
						x: rng.gen_range(0.0..=1.0),
						y: rng.gen_range(0.0..=1.0),
					},
				})
				.collect(),
		};

		for o in &ct.objects {
			s.bodies.push(Doodad {
				pos: o.position.to_cartesian(),
				sprite: SpriteTexture(o.sprite.to_owned()),
				size: o.size,
				parallax: o.parallax,
			});
		}

		return s;
	}

	pub fn get_sprites(&self) -> Vec<Sprite> {
		return self.bodies.iter().map(|x| x.get_sprite()).collect();
	}
}
